<?php

/**
 * Specific validation methods for data used in Hungary
 *
 * @category  Validate
 * @package   Validate_HU
 * @author    Tibor Balogh <baloghtibor@dolphinet.hu>
 * @author    Zoltan Koteles <koteleszoltan@dolphinet.hu>
 * @copyright 2008 Tibor Balogh, Zoltan Koteles
 * @license   http://www.opensource.org/licenses/bsd-license.php  new BSD
 * @link      http://www.dolphinet.hu
 */


/**
 * Data validation class for Hungary
 *
 * This class provides methods to validate:
 *  - Postal code (H-7090, 7090)
 *  - Identity card (123456AA, AA123456)
 *  - Tax number
 *  - Bank account number
 *  - SSN number (TAJ szam)
 *
 * @category Validate
 * @package  Validate_HU
 * @author   Tibor Balogh <baloghtibor@dolphinet.hu>
 * @author   Zoltan Koteles <koteleszoltan@dolphinet.hu>
 * @license  http://www.opensource.org/licenses/bsd-license.php  new BSD
 * @version  Release: @package_version@
 * @link     http://www.dolphinet.hu
 */
class Validate_HU
{
    /**
     * Validate a HU postcode
     *
     * @param string $postcode HU postcode to validate
     *
     * @return bool  true if postcode is ok false otherwise
     */
    function postalCode($postcode)
    {
        return (bool)preg_match('/^(H-)?\d{4}$/', $postcode);
    }

    /**
     * Identity card validation check
     *
     * @param string $number Hungarian identity card
     *
     * @return bool true if identity card 'seems' correct
     */
    function identityCard($number)
    {
        return (bool)preg_match('/^[A-Z]{2}\d{6}$|^\d{6}[A-Z]{2}$/i', $number);
    }

    /**
     * Validates Hungarian eight digit CDV number
     *
     * @param string $number number to check
     *
     * @access protected
     * @return bool
     */
    function _checkCDV($number)
    {
        $cdv     = intval(substr($number, 7, 1));
        $weights = array(9, 7, 3, 1, 9, 7, 3);
        $sum     = 0;
        for ($i = 0; $i < 7; $i++) {
            $sum += $weights[$i]*intval(substr($number, $i, 1));
        }
        return ((10 -($sum % 10)) % 10 == $cdv);
    }

    /**
     * Tax number validation check
     *
     * @param mixed $number Hungarian taxnumber
     * @param mixed $dashes uses dashes in number or not
     *
     * @access public
     * @return void
     */
    function taxNumber($number, $dashes = false)
    {
        $dash = $dashes ? "-" : "";
        if (!preg_match("/^\d{8}".$dash."\d".$dash."\d{2}$/", $number)) {
            return false;
        }
        return Validate_HU::_checkCDV(substr($number, 0, 8));
    }

    /**
     * Bank account validation check 
     *
     * @param string $number Hungarian bankaccount number
     * @param bool   $dashes Number contains 1 or 2 dashes, default is false
     *
     * @return bool true if bankaccount number 'seems' correct
     */
    function bankAccountNumber($number, $dashes = false)
    {
        $dash = $dashes ? "-" : "";
        if (!preg_match("/^\d{8}".$dash."\d{8}(".$dash."\d{8})?$/", $number)) {
            return false;
        }
        $number = str_replace("-", "", $number);

        return 
            Validate_HU::_checkCDV(substr($number, 0, 8)) &&
            Validate_HU::_checkCDV(substr($number, 8, 8)) &&
            ((strlen($number) == 16) || Validate_HU::_checkCDV(substr($number, 16, 8)));
    }

    /**
     * Validate SSN (TAJ szam)
     *
     * @param string $ssn ssn number
     *
     * @access public
     * @return bool
     */
    function ssn($ssn)
    {
        if (!preg_match("/^\d{9}$/", $ssn)) return false;

        $weights = array(3, 7);
        $sum     = 0;
        for ($i = 0; $i<8; $i++) {
            $sum += $weights[$i % 2] * intval(substr($ssn, $i, 1));
        }
        return substr($ssn, 8, 1) == ($sum % 10);
    }
}
?>
